'use strict';

const utils = require('./utils');

// private

module.exports = (options) => {
  const opts = options || {};
  opts.rules = opts.rules || {};
  opts.plugins = opts.plugins || [];

  if (!opts.prefix) {
    throw new TypeError('`options.prefix` should be specified');
  }

  // private

  /**
   * @param {string} ruleName
   * @return {boolean}
   */
  function isPluginRule(ruleName) {
    return options.plugins.some((plugin) => ruleName.indexOf(`${plugin}/`) !== -1);
  }

  /**
   * @param {string} location
   * @return {object} eslint plugin config
   */
  function loadConfig(location) {
    const config = require(location); // eslint-disable-line global-require
    return {
      env: config.env,
      extends: config.extends,
      parserOptions: config.parserOptions,
      plugins: [options.prefix],
      root: config.root,
      rules: utils.mapKeys(config.rules, (ruleName) => isPluginRule(ruleName)
        ? `${options.prefix}/${ruleName}`
        : ruleName),
      settings: config.settings,
    };
  }

  // create and expose plugin

  const configs = utils.mapValues(opts.configs, (value) => loadConfig(value));

  const rules = (opts.plugins || []).reduce((acc, pluginName) => {
    const plugin = require(`eslint-plugin-${pluginName}`); // eslint-disable-line global-require
    return Object.assign(acc, utils.mapKeys(plugin.rules, (ruleName) => `${pluginName}/${ruleName}`));
  }, {});

  return { configs, rules };
};
